#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

import os


SYSTEM_ENCODING = 'gbk' if os.name == 'nt' else 'utf-8'

def encode(s, encoding=None, errors='ignore'):
    us = s if isinstance(s, unicode) else decode(s)
    return us.encode(encoding or SYSTEM_ENCODING, errors)


def decode(s, encodings=['utf-8', 'gbk']):
    if isinstance(s, unicode):
        return s
    for enc in encodings:
        try:
            return unicode(s, enc)
        except:
            pass
    raise UnicodeDecodeError(','.join(encodings), "", 0, len(s or ''), "string: '%s'" % repr(s))


if __name__ == '__main__':
    print encode('Hello')


#! /usr/bin/env python
# -*- coding: utf-8 -*-

import urllib2
import shutil
import os
import subprocess
import commands
import sys


def http_download(url, target_path):
    """Download file to local
    Args:
        - url(string): url request path
        - target_path(string): download destination

    Raises:
        IOError
        urllib2.URLError
    """
    try:
        resp = urllib2.urlopen(url)
    except urllib2.URLError, e:
        if not hasattr(e, 'code'):
            raise
        resp = e
    if resp.code != 200:
        raise IOError("Request url(%s) expect 200 but got %d" %(url, resp.code))

    with open(target_path, 'wb') as f:
        shutil.copyfileobj(resp, f)
    return target_path



#-*- encoding: utf-8 -*-

import sys
import Tkinter as tk

import service
import keycode

if sys.platform == 'win32':
    from ctypes import wintypes, byref, windll
    import win32con

    def handle_hotkey(root, callback):
        msg = wintypes.MSG()
        if windll.user32.GetMessageA(byref(msg), None, 0, 0) != 0:
            if msg.message == win32con.WM_HOTKEY:
                if msg.wParam == 1:
                    print 'Hotkey triggered!'
                    callback()
        windll.user32.TranslateMessage(byref(msg))
        windll.user32.DispatchMessageA(byref(msg))
        root.after(1, handle_hotkey, root, callback)

    # hotkey map refs: https://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx
    # not yet used here.
    def register_hotkey(root, key, callback):
        key = key.split('-')
        mod = 0
        if 'Ctrl' in key:
            mod |= win32con.MOD_CONTROL
        if 'Shift' in key:
            mod |= win32con.MOD_SHIFT
        if 'Alt' in key:
            mod |= win32con.MOD_ALT
        key = key[-1].upper()
        assert key in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
        if windll.user32.RegisterHotKey(None, 1, mod, ord(key)) != 0:
            print("Hotkey registered!")
            handle_hotkey(root, callback)

else:
    def register_hotkey(root, key, callback):
        print 'Register hotkey failed.'

def main():
    service.start()

    root = tk.Tk()
    root.resizable(0, 0)
    root.title('STF Input')
    sv = tk.StringVar()

    if sys.platform == 'win32':
        backspace = '\x08'
    else:
        backspace = '\x7f'

    def send(event, sv=sv):
        char = event.char
        if not char:
            return
        text = sv.get()
        if char == '\r' and text: # use <Return> to input
            service.type(text)
            sv.set('')
            return
        if char == backspace and text: #  use <Backspace> to delete, <Del> not avaialable.
            sv.set('')
            return
        if char == '\x16': # skip <Ctrl-V>
            service.keyboard(char)
            sv.set('')
            return 'break'
        if char in keycode.KEYBOARD_KEYS or char in keycode.CTRLED_KEYS:
            service.keyboard(char)

    entry = tk.Entry(root, textvariable=sv)
    entry.pack()
    entry.focus_set()
    entry.bind('<Key>', send)

    state = [1]
    def toggle(root=root, entry=entry):
        if state[0] == 0:
            root.deiconify()
            entry.focus_set()
            state[0] = 1
        else:
            root.withdraw()
            state[0] = 0

    register_hotkey(root, 'Ctrl-Alt-Z', toggle) # not very well with IME

    try:
        root.mainloop()
    finally:
        service.stop()

if __name__ == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# License under MIT

from __future__ import absolute_import

import collections
import copy
import functools
import inspect
import os
import sys
import time
import traceback

import cv2
import aircv as ac
from uiautomator import AutomatorDeviceObject

from atx import base
from atx import consts
from atx import errors
from atx import imutils
from atx import logutils
from atx.base import nameddict
from atx.device import Pattern, Bounds, FindPoint


__dir__ = os.path.dirname(os.path.abspath(__file__))
log = logutils.getLogger(__name__)

_Condition = collections.namedtuple('WatchCondition', ['pattern', 'exists'])

class WatcherItem(object):
    """
    How to use, for
    example:

    with d.watch('xx') as w:
        w.on('button.png').on('enter.png').click()
        w.on('yes.png').on_not('exit.png').click()

    or

    w = d.watch('yy')
    w.on('button.png').on('enter.png').click()
    w.on('button.png').click('enter.png')
    w.run()

    # blueprint upgrade plan
    - conditions
        pattern
    - hooks
        functions
    """
    def __init__(self, watcher, rule):
        self._w = watcher
        self._r = rule

    def on(self, pattern):
        """Trigger when pattern exists"""
        self._r['conditions'].append(_Condition(pattern, True))
        return self

    # def on_not(self, pattern):
    #     """Trigger when pattern not exists"""
    #     self._conditions.append(_Condition(pattern, False))
    #     return self

    def do(self, func):
        """Trigger with function call
        Args:
            func: function which will called when object found. For example.

            def foo(event):
                print event.pos # (x, y) position
            
            w.on('kitty.png').do(foo)
        
        Returns:
            Watcher object

        Raises:
            SyntaxError
        """
        if not callable(func):
            raise SyntaxError("%s should be a function" % func)
        self._r['actions'].append(func)
        return self

    def click(self, *args, **kwargs):
        def _inner(event):
            if len(args) or len(kwargs):
                self._w._dev.click(*args, **kwargs)
            else:
                self._w._dev.click(*event.pos)
        return self.do(_inner)

    def click_image(self, *args, **kwargs):
        """ async trigger click_image """
        def _inner(event):
            return self._w._dev.click_image(*args, **kwargs)

        return self.do(_inner)

    def quit(self):
        def _inner(event):
            self._w._done = True
        return self.do(_inner)

class Watcher(object):
    Handler = collections.namedtuple('Handler', ['selector', 'action'])
    Event = collections.namedtuple('Event', ['selector', 'pos'])

    def __init__(self, device, name=None, timeout=None, raise_errors=True):
        self.name = name
        self.timeout = timeout
        self.raise_errors = raise_errors
        
        self._dev = device
        self._done = False
        self._watches = []

    def on(self, pattern):
        w = dict(
            conditions=[_Condition(pattern, True)],
            actions=[],
        )
        self._watches.append(w)
        return WatcherItem(self, w)

    def on_ui(self, text):
        w = dict(
            conditions=[_Condition(self._dev(text=text), True)],
            actions=[],
        )
        self._watches.append(w)
        return WatcherItem(self, w)

    def _do_hook(self, screen):
        # patterns = set()
        for rule in self._watches:
            conditions = rule['conditions']
            actions = rule['actions']

            if not actions:
                continue

            ok = True
            last_pos = None
            for condition in conditions:
                pos = self._match(condition.pattern, screen)
                if pos:
                    log.info("watch match: %s", condition)
                if bool(pos) != condition.exists:
                    ok = False
                    break
                if condition.exists:
                    last_pos = pos
            if ok:
                for fn in actions:
                    fn(self.Event(None, last_pos))
                break # FIXME(ssx): maybe need fallthrough, but for now, just simplfy it

    def run(self):
        # self._run = True
        start_time = time.time()
        while not self._done:
            screen = self._dev.screenshot()
            self._do_hook(screen)

            if self.timeout is not None:
                if time.time() - start_time > self.timeout:
                    if self.raise_errors:
                        raise errors.WatchTimeoutError("[%s] watch timeout %s" % (self.name, self.timeout,))
                    break
                sys.stdout.write("[%s] watching %4.1fs left: %4.1fs\r" %(self.name, self.timeout, self.timeout-time.time()+start_time))
                sys.stdout.flush()
        sys.stdout.write('\n')

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.run()

    def _match(self, selector, screen):
        ''' Find position for AtxPattern or UIAutomator Object

        Return:
            position(x, y) or None
        '''
        if isinstance(selector, Pattern) or isinstance(selector, basestring):
            ret = self._dev.match(selector, screen=screen)
            if ret is None:
                return
            log.debug('watch match: %s, confidence: %s', selector, ret)
            if not ret.matched:
                return
            return ret.pos
        elif isinstance(selector, AutomatorDeviceObject):
            if not selector.exists:
                return None
            info = selector.info['bounds']
            x = (info['left'] + info['right']) / 2
            y = (info['bottom'] + info['top']) / 2
            return (x, y)


Traceback = collections.namedtuple('Traceback', ['stack', 'exception'])
HookEvent = nameddict('HookEvent', ['flag', 'args', 'kwargs', 'retval', 'traceback', 'depth', 'is_before'])

def hook_wrap(event_type):
    def wrap(fn):
        @functools.wraps(fn)
        def _inner(*args, **kwargs):
            func_args = inspect.getcallargs(fn, *args, **kwargs)
            self = func_args.get('self')
            self._depth += 1

            def trigger(event):
                for (f, event_flag) in self._listeners:
                    if event_flag & event_type:
                        event.args = args[1:]
                        event.kwargs = kwargs
                        event.flag = event_type
                        event.depth = self._depth
                        f(event)
            
            _traceback = None
            _retval = None
            try:
                trigger(HookEvent(is_before=True))
                _retval = fn(*args, **kwargs)
                return _retval
            except Exception as e:
                _traceback = Traceback(traceback.format_exc(), e)
                raise
            finally:
                trigger(HookEvent(is_before=False, retval=_retval, traceback=_traceback))
                self._depth -= 1
        return _inner
    return wrap

class DeviceMixin(object):
    def __init__(self):
        self.image_match_method = consts.IMAGE_MATCH_METHOD_TMPL
        self.image_match_threshold = 0.8
        self._resolution = None
        self._bounds = None
        self._listeners = []
        self._depth = 0 # used for hook_wrap
        self.image_path = ['.']
        self.__last_screen = None
        self.__keep_screen = False

    @property
    def resolution(self):
        return self._resolution

    @resolution.setter
    def resolution(self, value):
        if value is None:
            self._resolution = None
        else:
            if not isinstance(value, tuple) or len(value) != 2:
                raise TypeError("Value should be tuple, contains two values")
            self._resolution = tuple(sorted(value))
    
    def _search_image(self, name, raise_error=True):
        image_path = base.search_image(name, self.image_path)
        if raise_error and image_path is None:
            raise IOError('image file not found: {}'.format(name))
        return image_path

    def pattern_open(self, image):
        if isinstance(image, Pattern):
            if image._image is None:
                image_path = self._search_image(image._name)
                image._image = imutils.open(image_path)
            return image
        elif isinstance(image, basestring):
            image_path = base.search_image(image, self.image_path)
            if image_path is None:
                raise IOError('image file not found: {}'.format(image))
            return Pattern(image_path, image=imutils.open(image_path))
        elif 'numpy' in str(type(image)):
            return Pattern('unknown', image=image)
        else:
            raise TypeError("Not supported image type: {}".format(type(image)))

    def delay(self, secs):
        """Delay some seconds
        Args:
            secs: float seconds

        Returns:
            self
        """
        secs = int(secs)
        for i in reversed(range(secs)):
            sys.stdout.write('\r')
            sys.stdout.write("sleep %ds, left %2ds" % (secs, i+1))
            sys.stdout.flush()
            time.sleep(1)
        sys.stdout.write("\n")
        return self

    def exists(self, pattern, screen=None):
        """Check if image exists in screen

        Returns:
            If exists, return FindPoint, or
            return None if result.confidence < self.image_match_threshold
        """
        ret = self.match(pattern, screen)
        if ret is None:
            return None
        if not ret.matched:
            return None
        return ret

    def wait(self, pattern, timeout=10.0):
        """Wait till pattern is found or time is out (default: 10s)."""
        t = time.time() + timeout
        while time.time() < t:
            ret = self.exists(pattern)
            if ret:
                return ret
            time.sleep(0.2)
        raise errors.ImageNotFoundError('Not found image %s' %(pattern,))

    def touch(self, x, y):
        """ Alias for click """
        self.click(x, y)

    def _cal_scale(self, pattern=None):
        scale = 1.0
        resolution = (pattern and pattern.resolution) or self.resolution
        if resolution is not None:
            ow, oh = sorted(resolution)
            dw, dh = sorted(self.display)
            fw, fh = 1.0*dw/ow, 1.0*dh/oh
            # For horizontal screen, scale by Y (width)
            # For vertical screen, scale by X (height)
            scale = fw if self.rotation in (1, 3) else fh
        return scale

    @property
    def bounds(self):
        if self._bounds is None:
            return None
        return self._bounds * self._cal_scale()
    
    def match(self, pattern, screen=None, threshold=None):
        """Check if image position in screen

        Args:
            - pattern: Image file name or opencv image object
            - screen: opencv image, optional, if not None, screenshot method will be called

        Returns:
            None or FindPoint, For example:

            FindPoint(pos=(20, 30), method='tmpl', confidence=0.801, matched=True)

            Only when confidence > self.image_match_threshold, matched will be True

        Raises:
            TypeError: when image_match_method is invalid
        """
        pattern = self.pattern_open(pattern)
        search_img = pattern.image

        pattern_scale = self._cal_scale(pattern)
        if pattern_scale != 1.0:
            search_img = cv2.resize(search_img, (0, 0), 
                fx=pattern_scale, fy=pattern_scale,
                interpolation=cv2.INTER_CUBIC)
        
        screen = screen or self.region_screenshot()
        threshold = threshold or pattern.threshold or self.image_match_threshold

        dx, dy = pattern.offset or (0, 0)
        # handle offset if percent, ex (0.2, 0.8)
        if 0 < dx <= 5:
            dx = pattern.image.shape[1] * dx # opencv object width
        if 0 < dy <= 5:
            dx = pattern.image.shape[0] * dy # opencv object height
        dx, dy = int(dx*pattern_scale), int(dy*pattern_scale)

        # image match
        screen = imutils.from_pillow(screen) # convert to opencv image
        match_method = self.image_match_method
        ret = None
        if match_method == consts.IMAGE_MATCH_METHOD_TMPL:
            ret = ac.find_template(screen, search_img)
        elif match_method == consts.IMAGE_MATCH_METHOD_SIFT:
            ret = ac.find_sift(screen, search_img, min_match_count=10)
        else:
            raise TypeError("Invalid image match method: %s" %(match_method,))

        if ret is None:
            return None
        (x, y) = ret['result']
        # fix by offset
        position = (x+dx, y+dy)
        if self.bounds:
            x, y = position
            position = (x+self.bounds.left, y+self.bounds.top)
        confidence = ret['confidence']

        matched = True
        if match_method == consts.IMAGE_MATCH_METHOD_TMPL:
            if confidence < threshold:
                matched = False
        elif match_method == consts.IMAGE_MATCH_METHOD_SIFT:
            matches, total = confidence
            if 1.0*matches/total > 0.5: # FIXME(ssx): sift just write here
                matched = True
        return FindPoint(position, confidence, match_method, matched=matched)

    def region(self, bounds):
        """Set region of the screen area
        Args:
            bounds: Bounds object

        Returns:
            A new AndroidDevice object

        Raises:
            TypeError
        """
        if not isinstance(bounds, Bounds):
            raise TypeError("region param bounds must be isinstance of Bounds")
        _d = copy.copy(self)
        _d._bounds = bounds
        return _d

    def keep_screen(self):
        """
        Freese screenshot, so all image functions will not take images, until call free_screen()
        """
        self.__last_screen = self.screenshot()
        self.__keep_screen = True
        return self
        
    def free_screen(self):
        """
        Unlock keep_screen()
        """
        self.__keep_screen = False
        return self

    def region_screenshot(self, filename=None):
        """ take part of the screenshot """
        screen = self.__last_screen if self.__keep_screen else self.screenshot()
        if self.bounds:
            screen = screen.crop(self.bounds)
        if filename:
            screen.save(filename)
        return screen

    def touch_image(self, *args, **kwargs):
        """ALias for click_image"""
        self.click_image(*args, **kwargs)

    def add_listener(self, fn, event_flags):
        """Listen event
        Args:
            - fn: function call when event happends
            - event_flags: for example
                EVENT_UIAUTO_CLICK | EVENT_UIAUTO_SWIPE

        Returns:
            None
        """
        self._listeners.append((fn, event_flags))

    def _trigger_event(self, event_flag, event):
        for (fn, flag) in self._listeners:
            if flag & event_flag:
                fn(event)

    def assert_exists(self, pattern, timeout=20.0):
        """Assert if image exists
        Args:
            - image: image filename # not support pattern for now
            - timeout (float): seconds

        Returns:
            self

        Raises:
            AssertExistsError
        """
        pattern = self.pattern_open(pattern)
        search_img = pattern.image
        # search_img = imutils.open(image)
        log.info('assert exists image: %s', pattern)
        start_time = time.time()
        while time.time() - start_time < timeout:
            point = self.match(search_img)
            if point is None:
                sys.stdout.write('.')
                sys.stdout.flush()
                continue
            if not point.matched:
                log.debug('Ignore confidence: %s', point.confidence)
                continue
            log.debug('assert pass, confidence: %s', point.confidence)
            sys.stdout.write('\n')
            break
        else:
            sys.stdout.write('\n')
            raise errors.AssertExistsError('image not found %s' %(pattern,))
            
    # TODO: need to add hook here
    def click_nowait(self, pattern, action='click'):
        """ Return immediately if no image found

        Args:
            - pattern (str or Pattern): filename or an opencv image object.
            - action (str): click or long_click

        Returns:
            Click point or None
        """
        point = self.match(pattern)
        if not point or not point.matched:
            return None

        func = getattr(self, action)
        func(*point.pos)
        return point

    @hook_wrap(consts.EVENT_CLICK_IMAGE)
    def click_image(self, pattern, timeout=20.0, action='click', safe=False):
        """Simulate click according image position

        Args:
            - pattern (str or Pattern): filename or an opencv image object.
            - timeout (float): if image not found during this time, ImageNotFoundError will raise.
            - action (str): click or long_click
            - safe (bool): if safe is True, Exception will not raise and return None instead.
        Returns:
            None

        Raises:
            ImageNotFoundError: An error occured when img not found in current screen.
        """
        pattern = self.pattern_open(pattern)
        log.info('click image: %s', pattern)
        start_time = time.time()
        found = False
        point = None
        while time.time() - start_time < timeout:
            point = self.match(pattern)
            if point is None:
                sys.stdout.write('.')
                sys.stdout.flush()
                continue

            log.debug('confidence: %s', point.confidence)
            if not point.matched:
                log.info('Ignore confidence: %s', point.confidence)
                continue
            
            func = getattr(self, action)
            func(*point.pos)

            found = True
            break
        sys.stdout.write('\n')

        if not found:
            if safe:
                log.info("Image(%s) not found, safe=True, skip", pattern)
                return None
            raise errors.ImageNotFoundError('Not found image %s' %(pattern,))

        # FIXME(ssx): maybe this function is too complex
        return point #collections.namedtuple('X', ['pattern', 'point'])(pattern, point)

    def watch(self, name='', timeout=None, raise_errors=True):
        """Return a new watcher
        Args:
            name: string watcher name
            timeout: watch timeout

        Returns:
            watcher object
        """
        w = Watcher(self, name, timeout, raise_errors)
        w._dev = self
        return w


if __name__ == '__main__':
    b = Bounds(1, 2, 3, 4)
    print b
    print b * 1.0



#!/usr/bin/env python
# coding: utf-8

import atx


def test_ios_screenshot():
	d = atx.connect(platform='ios')
	d.screen_rotation = 1
	print d.screenshot().save("yes.png")


if __name__ == '__main__':
	test_ios_screenshot()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

import os
import time

import wda
import subprocess32 as subprocess
from PIL import Image
from StringIO import StringIO

from atx.device.mixin import DeviceMixin, hook_wrap
from atx.device import Display
from atx import consts
from atx import ioskit

__dir__ = os.path.dirname(os.path.abspath(__file__))


class IOSDevice(DeviceMixin):
    def __init__(self, device_url): #udid=None):
        DeviceMixin.__init__(self)
        self.__device_url = device_url
        self.__display = None
        self.__scale = None
        
        self._wda = wda.Client(device_url)
        self._session = None
        # ioskit.Device.__init__(self, udid)

        # # xcodebuild -project  -scheme WebDriverAgentRunner -destination "id=1002c0174e481a651d71e3d9a89bd6f90d253446" test
        # # Test Case '-[UITestingUITests testRunner]' started.
        # xproj_dir = os.getenv('WEBDRIVERAGENT_DIR')
        # if not xproj_dir:
        #     raise RuntimeError("env-var WEBDRIVERAGENT_DIR need to be set")

        # proc = self._xcproc = subprocess.Popen(['/usr/bin/xcodebuild',
        #     '-project', 'WebDriverAgent.xcodeproj',
        #     '-scheme', 'WebDriverAgentRunner',
        #     '-destination', 'id='+self.udid, 'test'],
        #     stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd=xproj_dir, bufsize=1, universal_newlines=True)
        # for line in iter(proc.stdout.readline, b""):
        #     print 'STDOUT:', line.strip()
        #     if 'TEST FAILED' in line:
        #         raise RuntimeError("webdriver start test failed, maybe need to unlock the keychain, try\n" + 
        #             '$ security unlock-keychain ~/Library/Keychains/login.keychain')
        #     elif "Successfully wrote Manifest cache" in line:
        #         print 'GOOD ^_^, wait 5s'
        #         time.sleep(5.0)
        #         break

    # def __del__(self):
    #     if self._xcproc:
    #         print 'Terminate xcodebuild'
    #         self._xcproc.terminate()

    def start_app(self, bundle_id):
        """Start an application
        Args:
            - bundle_id: (string) apk bundle ID

        Returns:
            WDA session object
        """
        # if self._session is not None:
        #     self.stop_app()
        self._session = self._wda.session(bundle_id)
        return self._session

    def stop_app(self):
        if self._session is None:
            return
        self._session.close()
        self._session = None

    def __call__(self, *args, **kwargs):
        if self._session is None:
            raise RuntimeError("Need to call start_app before")
        return self._session(*args, **kwargs)

    def status(self):
        """ Check if connection is ok """
        return self._wda.status()

    @property
    def display(self):
        """ Get screen width and height """
        if not self.__display:
            self.screenshot()
        return self.__display

    @property
    def rotation(self):
        raise NotImplementedError()
        return self._wda.rotation

    def click(self, x, y):
        """Simulate click operation
        Args:
            x, y(int): position
        """
        if self._session is None:
            raise RuntimeError("Need to call start_app before")
        if not self.__scale:
            raw_size = self._session.window_size()
            self.__scale = self.display.width / int(round(min(raw_size['width'], raw_size['height'])))
        rx, ry = x/self.__scale, y/self.__scale
        self._session.tap(rx, ry)

    def home(self):
        """ Return to homescreen """
        return self._wda.home()

    def screenshot(self, filename=None):
        """Take a screenshot
        Args:
            - filename(string): file name to save

        Returns:
            PIL Image object
        """
        raw_png = self._wda.screenshot()
        img = Image.open(StringIO(raw_png))
        if filename:
            img.save(filename)
        if not self.__display:
            self.__display = Display(*sorted(img.size))
        return img



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# USAGE
# python -matx -s ESLKJXX gui

import argparse
import functools
import json
import sys
import inspect
from contextlib import contextmanager

import atx.androaxml as apkparse

from atx.cmds import run
from atx.cmds import iosdeveloper
from atx.cmds import install


def inject(func, kwargs):
    '''
    This is a black techonology of python. like golang martini inject

    Usage:
    allargs = dict(name='foo', message='body')
    def func(name):
        print 'hi', name

    inject(func, allargs)
    # will output
    # hi foo
    '''
    args = []
    for name in inspect.getargspec(func).args:
        args.append(kwargs.get(name))
    return func(*args)


def load_main(module_name):
    def _inner(parser_args):
        module_path = 'atx.cmds.'+module_name
        __import__(module_path)
        mod = sys.modules[module_path]
        pargs = vars(parser_args)
        return inject(mod.main, pargs)
    return _inner


def _apk_parse(args):
    (pkg_name, activity) = apkparse.parse_apk(args.filename)
    print json.dumps({
        'package_name': pkg_name,
        'main_activity': activity,
    }, indent=4)


def _version(args):
    import atx
    print atx.version


def main():
    ap = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    ap.add_argument("-s", "--serial", "--udid", required=False, help="Android serial or iOS unid")
    ap.add_argument("-H", "--host", required=False, default='127.0.0.1', help="Adb host")
    ap.add_argument("-P", "--port", required=False, type=int, default=5037, help="Adb port")

    subp = ap.add_subparsers()

    @contextmanager
    def add_parser(name):
        yield subp.add_parser(name, formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    with add_parser('tcpproxy') as p:
        p.add_argument('-l', '--listen', default=5555, type=int, help='Listen port')
        p.add_argument('-f', '--forward', default=26944, type=int, help='Forwarded port')
        p.set_defaults(func=load_main('tcpproxy'))

    with add_parser('gui') as p:
        p.add_argument('-p', '--platform', default='android', choices=('android', 'ios'), help='platform')
        p.set_defaults(func=load_main('tkgui'))

    with add_parser('record') as p:
        p.set_defaults(func=load_main('record'))

    with add_parser('minicap') as p:
        p.set_defaults(func=load_main('minicap'))

    with add_parser('apkparse') as p:
        p.add_argument('filename', help='Apk filename')
        p.set_defaults(func=_apk_parse)

    with add_parser('monkey') as p:
        p.set_defaults(func=load_main('monkey'))

    with add_parser('install') as p:
        p.add_argument('path', help='<apk file path | apk url path> (only support android for now)')
        p.add_argument('--start', action='store_true', help='Start app when app success installed')
        p.set_defaults(func=load_main('install'))

    with add_parser('screen') as p:
        p.add_argument('-s', '--scale', required=False, default=0.5, help='image scale')
        p.add_argument('--simple', action='store_true', help='disable interact controls')
        p.set_defaults(func=load_main('screen'))

    with add_parser('screencap') as p:
        p.add_argument('--scale', required=False, type=float, default=1.0, help='image scale')
        p.add_argument('-o', '--out', required=False, default='screenshot.png', help='output path')
        p.add_argument('-m', '--method', required=False, default='minicap', choices=('minicap', 'screencap'), help='screenshot method')
        p.set_defaults(func=load_main('screencap'))

    with add_parser('screenrecord') as p:
        p.add_argument('-o', '--output', default='out.avi', help='video output path')
        p.add_argument('--overwrite', action='store_true', help='overwrite video output file.')
        p.add_argument('--scale', type=float, default=0.5, help='image scale for video')
        p.add_argument('-q', '--quiet', dest='verbose', action='store_false', help='display screen while recording.')
        p.add_argument('--portrait', action='store_true', help='set video framesize to portrait instead of landscape.')
        p.set_defaults(func=load_main('screenrecord'))

    with add_parser('web') as p:
        p.add_argument('-b', '--no-browser', dest='open_browser', action='store_false', help='Not open browser')
        p.add_argument('--web-port', required=False, type=int, help='web listen port')
        p.set_defaults(func=load_main('webide'))

    with add_parser('run') as p:
        p.add_argument('-f', dest='config_file', default='atx.yml', help='config file')
        p.set_defaults(func=load_main('run'))

    with add_parser('version') as p:
        p.set_defaults(func=_version)

    with add_parser('info') as p:
        p.set_defaults(func=load_main('info'))

    args = ap.parse_args()
    args.func(args)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import pytest
import string

from airtest.image import auto as image

def test_image_locate_file_not_exists():
    with pytest.raises(IOError):
        image.locate_one_image('yy.png', 'zz.png')
    with pytest.raises(IOError):
        image.locate_one_image('testdata/yy.png', 'zz.png')

def test_locate_one_image():
    dirs = [p for p in os.listdir('testdata') if p.startswith('oneimg-')]
    for d in dirs:
        folder = os.path.join('testdata', d)

        EXPECT_FILE = os.path.join(folder, 'expect.txt')
        TRAIN_FILE = os.path.join(folder, 'train.png')
        if not os.path.exists(TRAIN_FILE):
            TRAIN_FILE = os.path.join(folder, 'train.jpg')

        assert os.path.exists(TRAIN_FILE)

        #
        # format: <filename> <left-up> <right-bottom>
        # eg: q1.png 100 200 40 20
        #
        if os.path.exists(EXPECT_FILE):
            for line in open(EXPECT_FILE):
                line = line.strip()
                if not line or line.startswith('#'):
                    continue
                name, extra = string.split(line, maxsplit=1)
                query_file = os.path.join(folder, name)
                print 'TEST EXISTS:', query_file, extra
                x0, y0, x1, y1 = map(int, extra.split())
                assert x0 < x1
                assert y0 < y1

                point = image.locate_one_image(TRAIN_FILE, query_file)
                assert point != None
                (x, y) = point
                assert x0 < x < x1
                assert y0 < y < y1

        NOEXISTS_FILE = os.path.join(folder, 'noexists.txt')
        if os.path.exists(NOEXISTS_FILE):
            for line in open(NOEXISTS_FILE):
                line = line.strip()
                if not line or line.startswith('#'):
                    continue
                filename = os.path.join(folder, line)
                print 'TEST NOT EXISTS:', filename
                point = image.locate_one_image(TRAIN_FILE, filename)
                assert point == None




#-*- encoding: utf-8 -*-

import os
import os.path
import json
import socket
import webbrowser
import tornado.ioloop
import tornado.web
import signal

from tornado.web import StaticFileHandler

__dir__ = os.path.dirname(os.path.abspath(__file__))

class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.redirect('/index.html')

class CaseHandler(tornado.web.RequestHandler):
    def initialize(self, casedir):
        self.casedir = casedir
        self.casepath = os.path.join(casedir, 'case.json')
        self.case = []
        if os.path.exists(self.casepath):
            with open(self.casepath) as f:
                self.case = json.load(f)

    def get(self, *args):
        self.write(json.dumps(self.case))
        self.finish()

    def post(self, *args):
        data = self.request.arguments
        with open(self.casepath, 'w') as f:
            json.dump(data, f)
        self.write(json.dumps({'success': True}))

class CaseRunnerHandler(tornado.web.RequestHandler):
    def initialize(self, casedir):
        pass

    def get(self, *args):
        pass

def get_valid_port():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('', 0))
    port = s.getsockname()[1]
    s.close()
    return port

def run(basedir):
    basedir = os.path.abspath(basedir)
    application = tornado.web.Application([
        (r'/', MainHandler),
        (r'/frames/(.*)', tornado.web.StaticFileHandler, {'path':os.path.join(basedir, 'frames')}),
        (r'/case(.*)', CaseHandler, {'casedir':os.path.join(basedir, 'case')}),
        (r'/run(.*)', CaseRunnerHandler, {'casedir': os.path.join(basedir, 'case')}),
        (r'/(.*)', tornado.web.StaticFileHandler, {'path':'site'}),
    ], autoreload=True, static_hash_cache=False)

    # port = get_valid_port()
    # webbrowser.open('http://127.0.0.1:%s' % port, new=2)
    port = 8000

    application.listen(port)
    print 'Listen on', port
    print 'CaseDir:', basedir
    tornado.ioloop.IOLoop.instance().start()

if __name__ == '__main__':
    run('testcase')



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import random

import atx
import cv2
from atx import imutils


# codeskyblue by 2016-05-03
# 过新手流程倒是挺容易的，不过还是有好多东西识别不出来，比如退出键，阴影下的属性切换键
# 还需要结合其他的方法继续去弄

def choose_point(frame):
    h, w = frame.shape[:2]
    framesize = h*w
    minarea = framesize/1000
    print 'minarea:', minarea

    hsv = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV)
    cv2.imshow('hsv', hsv)
    H, S, V = cv2.split(hsv)
    V[V<200] = 0
    # cv2.imshow('H', H)
    # cv2.imshow('S', S)
    cv2.imshow('V', V)

    mask = V
    mask = cv2.dilate(mask, None, iterations=2)
    # mask = cv2.erode(mask, None, iterations=2)
    cv2.imshow('mask', mask)

    (cnts, _) = cv2.findContours(mask.copy(), cv2.RETR_EXTERNAL,
        cv2.CHAIN_APPROX_SIMPLE)

    points = []
    for c in cnts:
        # if the contour is too small, ignore it
        area = cv2.contourArea(c)
        print area
        if area < minarea:
            continue
        if area > framesize/2:
            continue
        # if cv2.contourArea(c) < minarea:
            # continue

        # compute the bounding box for the contour, draw it on the frame,
        # and update the text
        # 计算轮廓的边界框，在当前帧中画出该框
        (x, y, w, h) = cv2.boundingRect(c)
        points.append((x, y, w, h))
        cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)
        # text = "Occupied"

    # random pick point
    if not points:
        return None
    pt = random.choice(points)
    (x, y, w, h) = pt
    cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 0, 255), 2)
    cv2.imshow('frame', frame)
    return (x+w/2, y+h/2)
    # cv2.waitKey(0)


def main(serial=None, host=None, port=None):
    d = atx.connect(serial, host=host, port=port)
    while True:
        pilimg = d.screenshot()
        cv2img = imutils.from_pillow(pilimg)
        # cv2img = cv2.imread('tmp.png')
        # cv2.imwrite('tmp.png', cv2img)
        cv2img = cv2.resize(cv2img, fx=0.5, fy=0.5, dsize=(0, 0))
        pt = choose_point(cv2img)
        print 'click:', pt
        if pt:
            x, y = pt
            d.click(2*x, 2*y)
        cv2.waitKey(100)
        # import time
        # time.sleep(0.1)


#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Description: minicap setup scripts,
# Usage: python minicap_setup.py -s serialno -H host -P port
# Author: ydbn2153
# Created:  ydbn2153 <2016-03-15>
# Modified: hzsunshx <2016-03-19>

import argparse
import os
import sys
import shutil
import subprocess
import tempfile
import urllib
import functools

from atx import logutils
from atx.cmds.utils import http_download

logger = logutils.getLogger('minicap')


def log(*args):
    logger.info(*args)


def check_output(cmdstr, shell=True):
    output = subprocess.check_output(cmdstr, stderr=subprocess.STDOUT, shell=shell)
    return output


def run_adb(*args, **kwargs):
    cmds = ['adb']
    serialno = kwargs.get('serialno', None)
    if serialno:
        cmds.extend(['-s', serialno])
    host = kwargs.get('host')
    if host:
        cmds.extend(['-H', host])
    port = kwargs.get('port')
    if port:
        cmds.extend(['-P', str(port)])
    cmds.extend(args)
    cmds = map(str, cmds)
    cmdline = subprocess.list2cmdline(cmds)
    try:
        return check_output(cmdline, shell=False)
    except Exception, e:
        raise EnvironmentError('run cmd: {} failed. {}'.format(cmdline, e))


def main(serialno=None, host=None, port=None):
    logger.info("Minicap install started!")
    
    adb = functools.partial(run_adb, serialno=serialno, host=host, port=port)

    # Figure out which ABI and SDK
    logger.info("Make temp dir ...")
    tmpdir = tempfile.mkdtemp(prefix='ins-minicap-')
    logger.debug(tmpdir)
    try:
        logger.info("Retrive device information ...")
        abi = adb('shell', 'getprop', 'ro.product.cpu.abi').strip()
        sdk = adb('shell', 'getprop', 'ro.build.version.sdk').strip()

        logger.info("Downloading minicap.so ....")
        url = "https://github.com/openstf/stf/raw/master/vendor/minicap/shared/android-"+sdk+"/"+abi+"/minicap.so"
        target_path = os.path.join(tmpdir, 'minicap.so')
        http_download(url, target_path)
        logger.info("Push data to device ....")
        adb('push', target_path, '/data/local/tmp')
        
        logger.info("Downloading minicap ....")
        url = "https://github.com/openstf/stf/raw/master/vendor/minicap/bin/"+abi+"/minicap"
        target_path = os.path.join(tmpdir, 'minicap')
        http_download(url, target_path)
        logger.info("Push data to device ....")
        adb('push', target_path, '/data/local/tmp')
        adb('shell', 'chmod', '0755', '/data/local/tmp/minicap')

        logger.info("Checking [dump device info] ...")
        print adb('shell', 'LD_LIBRARY_PATH=/data/local/tmp /data/local/tmp/minicap -i')
        logger.info("Minicap install finished !")

        logger.info("Downloading minitouch ....")
        url = "https://github.com/openstf/stf/raw/master/vendor/minitouch/"+abi+"/minitouch"
        target_path = os.path.join(tmpdir, 'minitouch')
        http_download(url, target_path)
        logger.info("Push data to device ....")
        adb('push', target_path, '/data/local/tmp')
        adb('shell', 'chmod', '0755', '/data/local/tmp/minitouch')

        logger.info("Checking [dump device info] ...")
        print adb('shell', '/data/local/tmp/minitouch -h')
        logger.info("Minitouch install finished !")

    except Exception, e:
        logger.error(e)
    finally:
        if tmpdir:
            logger.info("Cleaning temp dir")
            shutil.rmtree(tmpdir)
    

if __name__ == "__main__":
    parser = argparse.ArgumentParser("cli")
    parser.add_argument("-s", "--serialno", help="serialno of device", default=None)
    parser.add_argument("-H", "--host", help="host of remote device", default=None)
    parser.add_argument("-P", "--port", help="port of remote device", default=None)
    args = parser.parse_args(sys.argv[1:])
    main(serialno=args.serialno, host=args.host, port=args.port)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import time

import pytest
import atx

d = atx.connect(platform='dummy')

def setup_function(f):
    d.resolution = (1280, 720)

def test_setget_resolution():
    assert d.resolution == (720, 1280)

    d.resolution = None # None is also OK to set
    assert d.resolution is None

    d.resolution = (200, 400)
    assert d.resolution == (200, 400)

    with pytest.raises(TypeError):
        d.resolution = [1, 3]
    with pytest.raises(TypeError):
        d.resolution = 720
    assert d.resolution == (200, 400)
    
def teardown_function(f):
    print 'teardown'
    
def test_screenshot():
    screen = d.screenshot()
    assert screen is not None

def test_hook_screenshot():
    called = [False]

    def hook(event):
        print 'event', event
        called[0] = True

    d.add_listener(hook, atx.EVENT_SCREENSHOT)
    d.screenshot()
    assert called[0] == True

#def test_cloudtest_hook():
#    cloudtest.record_operation(d)
#    d.screenshot()

def test_region_screenshot():
    nd = d.region(atx.Bounds(100, 100, 600, 300))
    rs = nd.region_screenshot()
    assert rs is not None
    assert rs.size == (500, 200)

def test_assert_exists():
    d.assert_exists('media/system-app.png')

    with pytest.raises(atx.AssertExistsError):
        d.assert_exists('media/haima.png', timeout=0.1)

def test_click():
    d.click(50, 70)
    assert d.last_click == (50, 70)

def test_click_image():
    """ require aircv installed """
    d.click_image('media/system-app.png')
    assert d.last_click == (139, 299)

def test_click_image_offset1():
    d.click_image(atx.Pattern('media/system-app.png'))
    assert d.last_click == (139, 299)

def test_click_image_offset2():
    d.click_image(atx.Pattern('media/system-app.png', offset=(10, 10)))
    assert d.last_click == (149, 309)    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# extention for http://gt.qq.com/
# reference doc http://gt.qq.com/docs/a/UseGtWithBroadcast.txt
#
# Experimental, maybe change in the future
# Created by <hzsunshx> 2016-06-12

import functools


class GT(object):
    def __init__(self, d):
        self.d = d
        self._broadcast = functools.partial(self.d.adb_device.shell, 'am', 'broadcast', '-a')
        self._package_name = None

    def start_test(self, package_name, cpu=True, net=True, pss=True):
        self._package_name = package_name

        broadcast = self._broadcast
        # 1. start app
        self.quit() # reset gt
        self.d.start_app('com.tencent.wstt.gt')#, 'com.tencent.wstt.gt.activity.GTMainActivity')
        # 2. set test package name
        broadcast('com.tencent.wstt.gt.baseCommand.startTest', '--es', 'pkgName', package_name)
        # 3. set collect params
        if cpu:
            broadcast('com.tencent.wstt.gt.baseCommand.sampleData', '--ei', 'cpu', '1')
        if net:
            broadcast('com.tencent.wstt.gt.baseCommand.sampleData', '--ei', 'net', '1')
        if pss:
            broadcast('com.tencent.wstt.gt.baseCommand.sampleData', '--ei', 'pss', '1')

        # 4. switch back to app
        self.d.start_app(package_name)

    def stop_and_save(self):
        self._broadcast('com.tencent.wstt.gt.baseCommand.endTest', '--es', 'saveFolderName', self._package_name,
            '--es', 'desc', 'Result_of_GT')
        print 'Run\n$ adb pull /sdcard/GT/GW/{pkgname}/{version}/{pkgname}'.format(pkgname=self._package_name, version='unknow')

    def quit(self):
        self._broadcast('com.tencent.wstt.gt.baseCommand.exitGT')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""This module is to make mobile test more easily
"""

from __future__ import absolute_import

import os
import sys
import signal

import pkg_resources
try:
    version = pkg_resources.get_distribution("atx").version
except pkg_resources.DistributionNotFound:
    version = 'unknown'

from atx.consts import *
from atx.errors import *
from atx.device import Pattern, Bounds


def connect(*args, **kwargs):
    """Connect to a device, and return its object
    Args:
        platform: string one of <android|ios|windows>
        
    Returns:
        None

    Raises:
        SyntaxError, EnvironmentError
    """
    platform = kwargs.pop('platform', os.getenv('ATX_PLATFORM') or 'android')

    cls = None
    if platform == 'android':
        os.environ['JSONRPC_TIMEOUT'] = "10" # default is 90s which is too long.
        devcls = __import__('atx.device.android')
        cls = devcls.device.android.AndroidDevice
    elif platform == 'windows':
        devcls = __import__('atx.device.windows')
        cls = devcls.device.windows.WindowsDevice
    elif platform == 'ios':
        devcls = __import__('atx.device.ios_webdriveragent')
        cls = devcls.device.ios_webdriveragent.IOSDevice
    elif platform == 'dummy': # for py.test use
        devcls = __import__('atx.device.dummy')
        cls = devcls.device.dummy.DummyDevice
    
    if cls is None:
        raise SyntaxError('Platform: %s not exists' % platform)
    c = cls(*args, **kwargs)
    return c


# def _sig_handler(signum, frame):
#     print >>sys.stderr, 'Signal INT catched !!!'
#     sys.exit(1)
# signal.signal(signal.SIGINT, _sig_handler)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import json
import atx


def main(serial, host, port):
    d = atx.connect(serial, host=host, port=port)
    props = d.properties
    (w, h) = d.display
    info = {
        'serial': d.serial,
        'product.model': props['ro.product.model'],
        'product.brand': props.get('ro.product.brand'),
        'sys.country': props.get('persist.sys.country'),
        'display': '%dx%d' % (w, h),
        'version.sdk': int(props.get('ro.build.version.sdk', 0)),
        'version.release': props.get('ro.build.version.release'),
        'product.cpu.abi': props.get('ro.product.cpu.abi'),
    }
    print json.dumps(info, indent=4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Some code reference
# https://github.com/jrosebr1/imutils
#

import re
import os
import sys
import cv2
import base64

import numpy as np
from PIL import Image
from StringIO import StringIO

# import any special Python 2.7 packages
if sys.version_info.major == 2:
    from urllib import urlopen

# import any special Python 3 packages
elif sys.version_info.major == 3:
    from urllib.request import urlopen


__sys_open = open


def _open_data_url(data, flag=cv2.IMREAD_COLOR):
    pos = data.find('base64,')
    if pos == -1:
        raise IOError("data url is invalid, head %s" % data[:20])

    pos += len('base64,')
    raw_data = base64.decodestring(data[pos:])
    image = np.asarray(bytearray(raw_data), dtype="uint8")
    image = cv2.imdecode(image, flag)
    return image


def open(image):
    '''
    Args:
        - image: support many type. filepath or url or data:image/png:base64
    Return:
        Pattern
    Raises
        IOError
    '''
    if isinstance(image, basestring):
        name = image
        if name.startswith('data:image/'):
            return _open_data_url(name)
        if re.match(r'^https?://', name):
            return url_to_image(name)
        if os.path.isfile(name):
            img = cv2.imread(name)
            if img is None:
                raise IOError("Image format error: %s" % name)
            return img
        raise IOError("Open image(%s) not found" % name)

    return image

def open_as_pillow(filename):
    """ This way can delete file immediately """
    with __sys_open(filename, 'rb') as f:
        data = StringIO(f.read())
        return Image.open(data)


def from_pillow(pil_image):
    """ Convert from pillow image to opencv """
    # convert PIL to OpenCV
    pil_image = pil_image.convert('RGB')
    cv2_image = np.array(pil_image)
    # Convert RGB to BGR 
    cv2_image = cv2_image[:, :, ::-1].copy()
    return cv2_image


def to_pillow(image):
    return Image.fromarray(image[:, :, ::-1].copy())

    # There is another way
    # img_bytes = cv2.imencode('.png', image)[1].tostring()
    # return Image.open(StringIO(img_bytes))

def url_to_image(url, flag=cv2.IMREAD_COLOR):
    """ download the image, convert it to a NumPy array, and then read
    it into OpenCV format """
    resp = urlopen(url)
    image = np.asarray(bytearray(resp.read()), dtype="uint8")
    image = cv2.imdecode(image, flag)
    return image


def crop(image, left=0, top=0, right=None, bottom=None):
    (h, w) = image.shape[:2]
    if bottom is None:
        bottom = h
    if right is None:
        right = w
    return image[top:bottom, left:right]

def diff_rect(img1, img2, pos=None):
    """find counters include pos in differences between img1 & img2 (cv2 images)"""
    diff = cv2.absdiff(img1, img2)
    diff = cv2.GaussianBlur(diff, (3, 3), 0)
    edges = cv2.Canny(diff, 100, 200)
    _, thresh = cv2.threshold(edges, 0, 255, cv2.THRESH_BINARY)
    contours, _ = cv2.findContours(thresh, cv2.RETR_TREE, cv2.CHAIN_APPROX_NONE)
    if not contours:
        return None
    contours.sort(key=lambda c: len(c))
    # no pos provide, just return the largest different area rect
    if pos is None:
        cnt = contours[-1]
        x0, y0, w, h = cv2.boundingRect(cnt)
        x1, y1 = x0+w, y0+h
        return (x0, y0, x1, y1)
    # else the rect should contain the pos
    x, y = pos
    for i in range(len(contours)):
        cnt = contours[-1-i]
        x0, y0, w, h = cv2.boundingRect(cnt)
        x1, y1 = x0+w, y0+h
        if x0 <= x <= x1 and y0 <= y <= y1:
            return (x0, y0, x1, y1)

if __name__ == '__main__':
    # image = open('https://ss0.bdstatic.com/5aV1bjqh_Q23odCf/static/superman/img/logo/bd_logo1_31bdc765.png')
    image = open('baidu.png')
    image = open(image)
    # cv2.imwrite('baidu.png', image)
    print image.shape
    image = crop(image, bottom=200, top=100, left=50, right=200)
    print image.shape
    cv2.imwrite('tmp.png', image)
    # to_pillow(image).save('b2.png')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# parse and run atx.yml
#
# # example of atx.yml
# #
# installation: http://example.com/demo.apk
# script:
# - python test1.py
# - python test2.py
# notification:
#   popo:
#   - someone@example.com
#
from __future__ import absolute_import

import os
import sys
import json
from argparse import Namespace

import yaml
import subprocess32 as subprocess


def json2obj(data):
    return json.loads(json.dumps(data), object_hook=lambda d: Namespace(**d))


def prompt(message):
    print '>>>', message


def must_exec(*cmds, **kwargs):
    prompt("Exec %s" % cmds)
    shell = kwargs.get('shell', False)
    cmdline = cmds[0] if shell else subprocess.list2cmdline(cmds)
    ret = os.system(cmdline)
    if ret != 0:
        raise SystemExit("Execute '%s' error" % cmdline)


def install(src):
    prompt("Install")
    must_exec('python', '-matx', 'install', src)


def runtest(scripts):
    prompt("Run scripts")
    for script in scripts:
        must_exec(script, shell=True)


def notify_popo(users, message):
    prompt("Notify popo users")
    print 'Skip, todo'
    for user in users:
        pass
    # maybe should not put code here
    # print users, message


def main(config_file='atx.yml'):
    if not os.path.exists(config_file):
        sys.exit('config file (%s) not found.' % config_file)

    with open(config_file, 'rb') as f:
        cfg = json2obj(yaml.load(f))
    
    try:
        if hasattr(cfg, 'installation'):
            install(cfg.installation)

        if hasattr(cfg, 'script'):
            if isinstance(cfg.script, basestring):
                scripts = [cfg.script]
            else:
                scripts = cfg.script
            runtest(scripts)
    finally:
        if hasattr(cfg, 'notification'):
            if hasattr(cfg.notification, 'popo'):
                notify_popo(cfg.notification.popo, 'hi')


if __name__ == '__main__':
    main('atx.yml')


#-*- encoding: utf-8 -*-

import sys
import time
import service

def on_battery(info):
    print 'on battery info', info

def on_rotation(rot):
    print 'on rotation', rot

def test_service():
    serial = "DU2SSE1467010532"; #hwH60

    from random import randint
    import stfwire_pb2 as wire

    reqs = [
        (wire.GET_VERSION, wire.GetVersionRequest()),
        (wire.DO_IDENTIFY, wire.DoIdentifyRequest(serial=serial)),
        (wire.DO_ADD_ACCOUNT_MENU, wire.DoAddAccountMenuRequest()),
        (wire.DO_REMOVE_ACCOUNT, wire.DoRemoveAccountRequest(type="nsdfjslfs")),
        (wire.GET_ACCOUNTS, wire.GetAccountsRequest(type="root")),
        (wire.GET_BROWSERS, wire.GetBrowsersRequest()),
        (wire.GET_CLIPBOARD, wire.GetClipboardRequest(type=wire.TEXT)),
        (wire.GET_DISPLAY, wire.GetDisplayRequest(id=0)),
        (wire.GET_PROPERTIES, wire.GetPropertiesRequest(properties=["ro.product.device"])),
        (wire.GET_RINGER_MODE, wire.GetRingerModeRequest()),
        (wire.GET_SD_STATUS, wire.GetSdStatusRequest()),
        (wire.GET_WIFI_STATUS, wire.GetWifiStatusRequest()),
        (wire.SET_CLIPBOARD, wire.SetClipboardRequest(type=wire.TEXT, text="hello world")),
        (wire.SET_KEYGUARD_STATE, wire.SetKeyguardStateRequest(enabled=False)),
        (wire.SET_RINGER_MODE, wire.SetRingerModeRequest(mode=wire.VIBRATE)),
        (wire.SET_WAKE_LOCK, wire.SetWakeLockRequest(enabled=False)),
        (wire.SET_WIFI_ENABLED, wire.SetWifiEnabledRequest(enabled=True)),
        (wire.SET_MASTER_MUTE, wire.SetMasterMuteRequest(enabled=True)),
    ]

    total = len(reqs)
    idx = 0
    queue = service.service_queue
    pack = service.pack

    service.start_stf_service()
    service.listen_service()

    while True:

        if randint(1, 10) < 3 and idx < total:
            mtype, request = reqs[idx]
            msg = pack(mtype, request, idx)
            queue.put(msg)
            idx += 1
        time.sleep(1)

if sys.platform == 'win32':
    import msvcrt
    def getchar():
        return msvcrt.getch()
else:
    import tty
    import termios

    def getchar():
        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        try:
            tty.setraw(sys.stdin.fileno())
            ch = sys.stdin.read(1)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
        return ch

def test_type():
    import locale
    _, encoding = locale.getdefaultlocale()
    buf = ''
    while True:
        ch = getchar()
        if ch == '\x03': # Ctrl+C
            break
        buf += ch
        try:
            text = buf.decode(encoding)
        except:
            pass
        else:
            print 'try to input', repr(text)
            if len(buf) == 1:
                service.keyboard(buf)
            else:
                service.type(text)
            buf = ''

def test_agent():
    service.start_stf_agent(restart=True)
    service.listen_agent()

    print 'KEYCODE_HOME'
    service.keyevent('KEYCODE_HOME')
    #service.wake()

    print 'test ascii_type Ctrl+C to stop'
    while True:
        ch = getchar()
        print 'try to input', repr(ch)
        if ch == '\x03': # Ctrl+C
            break
        continue
        service.ascii_type(ch)

    print 'test keyboard Ctrl+C to stop'
    while True:
        ch = getchar()
        print 'try to input', repr(ch)
        if ch == '\x03': # Ctrl+C
            break
        continue
        service.keyboard(ch)

    #service.stop()
    
def testall():
    service.start()

    service.on_battery_event(on_battery)
    service.on_rotation_event(on_rotation)

    service.identify()
    time.sleep(2)
    service.keyevent('KEYCODE_HOME')
    time.sleep(2)

    print 'wifi is', service.get_wifi_status()
    print 'disable', service.set_wifi_enabled(False)
    print 'wifi is', service.get_wifi_status()
    print 'enable', service.set_wifi_enabled(True)
    print 'wifi is', service.get_wifi_status()
    time.sleep(1)

    print 'set rotation'
    print service.set_rotation(1)
    time.sleep(1)
    print service.set_rotation(2)
    time.sleep(1)
    print service.set_rotation(3)
    time.sleep(1)
    print service.set_rotation(0)
    time.sleep(1)

    print 'display', service.get_display()
    time.sleep(1)

    print 'test type, please input'
    test_type()

    service.stop()

if __name__ == '__main__':
    #test_service()
    #test_agent()
    testall()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import atexit
import os
import re
import subprocess32 as subprocess
import shutil
import sys
import time
import tempfile
import urllib2

import tqdm
import atx.androaxml as apkparse

from atx import logutils
from atx import adbkit
from atx.cmds import utils


log = logutils.getLogger('install')
DEFAULT_REMOTE_PATH = '/data/local/tmp/_atx_tmp.apk'
__apks = dict(
    utf7ime='http://o8oookdsx.qnssl.com/android_unicode_ime-debug.apk')


def clean(tmpdir):
    log.info('Remove temp directory')
    shutil.rmtree(tmpdir)


def adb_pushfile(adb, filepath, remote_path):
    filesize = os.path.getsize(filepath)
    pb = tqdm.tqdm(unit='B', unit_scale=True, total=filesize)
    p = adb.raw_cmd('push', filepath, remote_path)

    while True:
        try:
            p.wait(0.5)
        except subprocess.TimeoutExpired:
            pb.n = get_file_size(adb, remote_path)
            pb.refresh()
            # log.info("Progress %dM/%dM", get_file_size(remote_path) >>20, filesize >>20)
            pass
        except (KeyboardInterrupt, SystemExit):
            p.kill()
            raise
        except:
            raise
        else:
            # log.info("Success pushed into device")
            break
    pb.close()


def get_file_size(adb, remote_path):
    try:
        output = adb.run_cmd('shell', 'ls', '-l', remote_path)
        m = re.search(r'\s(\d+)', output)
        if not m:
            return 0
        return int(m.group(1))
    except subprocess.CalledProcessError as e:
        log.warn("call error: %s", e)
        time.sleep(.1)
        return 0


def adb_remove(adb, path):
    p = adb.raw_cmd('shell', 'rm', path)
    stdout, stderr = p.communicate()
    if stdout or stderr:
        log.warn('%s\n%s', stdout, stderr)


def adb_install(adb, remote_path):
    stdout = adb.run_cmd('shell', 'pm', 'install', '-rt', remote_path)
    # stdout, _ = p.communicate()
    if stdout.find('Success') == -1:
        raise IOError("Adb install failed: %s" % stdout)


def adb_must_install(adb, remote_path, package_name):
    try:
        adb_install(adb, remote_path)
    except IOError:
        log.info("Remove already installed app: %s", package_name)
        adb.raw_cmd('uninstall', package_name).wait()
        adb_install(adb, remote_path)


def main(path, serial=None, host=None, port=None, start=False):
    adb = adbkit.Client(host, port).device(serial) #adbutils.Adb(serial, host, port)
    
    # use qiniu paths
    if __apks.get(path):
        path = __apks.get(path)

    if re.match(r'^https?://', path):
        tmpdir = tempfile.mkdtemp(prefix='atx-install-')
        log.info("Create temp directory: %s", tmpdir)
        
        # FIXME(ssx): will not called when Ctrl+C pressed in windows git-bash
        atexit.register(clean, tmpdir)

        urlpath = path
        target = os.path.join(tmpdir, '_tmp.apk')
        path = target
        log.info("Download from: %s", urlpath)
        utils.http_download(urlpath, target)

    package_name, main_activity = apkparse.parse_apk(path)
    log.info("APK package name: %s", package_name)
    log.info("APK main activity: %s", main_activity)

    log.info("Push file to android device")
    adb_pushfile(adb, path, DEFAULT_REMOTE_PATH)

    log.info("Install ..., will take a few seconds")
    adb_must_install(adb, DEFAULT_REMOTE_PATH, package_name)
    log.info("Remove _tmp.apk")
    adb_remove(adb, DEFAULT_REMOTE_PATH)

    if start:
        log.info("Start app '%s'" % package_name)
        adb.raw_cmd('shell', 'am', 'start', '-n', package_name+'/'+main_activity).wait()
    log.info("Success")


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import cv2
import time
import numpy as np
import matplotlib.pyplot as plt
import scipy.fftpack
from cv2 import cv

from atx.record.monkey import Monkey, StupidMonkey, RandomContourMonkey
from atx.device.android_minicap import AndroidDeviceMinicap

def _binary_array_to_hex(arr):
    """
    internal function to make a hex string out of a binary array
    """
    h = 0
    s = []
    for i, v in enumerate(arr.flatten()):
        if v: 
            h += 2**(i % 8)
        if (i % 8) == 7:
            s.append(hex(h)[2:].rjust(2, '0'))
            h = 0
    return "".join(s)

def ahash(img, size=8):
    '''average hash'''
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    mat = cv2.resize(img, (size, size))
    avg = mat.mean()
    arr = mat > avg
    return arr
    # return _binary_array_to_hex(arr)


def phash(img, size=8, factor=4):
    '''perceptual Hash'''
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    mat = cv2.resize(img, (size*factor, size*factor))/1.0
    dct = scipy.fftpack.dct(scipy.fftpack.dct(mat, axis=0), axis=1)
    dctlowfreq = dct[:size, :size]
    med = np.median(dctlowfreq)
    arr = dctlowfreq > med
    return arr
    # return _binary_array_to_hex(arr)

def dhash(img, size=8):
    '''difference hash'''
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    mat = cv2.resize(img, (size+1, size))
    arr = mat[1:, :] > mat[:-1,:]
    return arr
    # return _binary_array_to_hex(arr)


## 尝试判断场景
## 尝试识别UI层(静止不动层) 
# ShiTomasi corner detection & Lucas Kanada optical flow
## 识别2D/3D运动
# Meanshift & Camshift
def test_features():
    from atx.device.android_minicap import AndroidDeviceMinicap
    cv2.namedWindow("preview")
    d = AndroidDeviceMinicap()

    # r, h, c, w = 200, 100, 200, 100
    # track_window = (c, r, w, h)
    # oldimg = cv2.imread('base1.png')
    # roi = oldimg[r:r+h, c:c+w]
    # hsv_roi = cv2.cvtColor(roi, cv2.COLOR_BGR2HSV)
    # mask = cv2.inRange(hsv_roi, 0, 255)
    # roi_hist = cv2.calcHist([hsv_roi], [0], mask, [180], [0,180])
    # cv2.normalize(roi_hist, roi_hist, 0, 255, cv2.NORM_MINMAX)
    # term_cirt = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT,  10, 1)


    while True:
        try:
            w, h = d._screen.shape[:2]
            img = cv2.resize(d._screen, (h/2, w/2))
            cv2.imshow('preview', img)

            hist = cv2.calcHist([img], [0], None, [256], [0,256])
            plt.plot(plt.hist(hist.ravel(), 256))
            plt.show()
            # if img.shape == oldimg.shape:
            #     # hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
            #     # ret, track_window = cv2.meanShift(hsv, track_window, term_cirt)
            #     # x, y, w, h = track_window
            #     cv2.rectangle(img, (x, y), (x+w, y+h), 255, 2)
            #     cv2.imshow('preview', img)
            # # cv2.imshow('preview', img)
            cv2.waitKey(1)
        except KeyboardInterrupt:
            break

    cv2.destroyWindow('preview')

def test_hist():
    files = ('base1.png', 'base2.png')
    for i in range(len(files)):
        img = cv2.imread(files[i])
        # hist = cv2.calcHist([img], [0], None, [256], [0,256])
        for j in range(3):
            plt.subplot(2, 3, 3*i+j+1)
            plt.hist(img[:,:,j].flatten(), 64)
            plt.title('%s-%d' % (files[i], j))
    plt.show()


# def similarize(img, r=5):
#     weights = [60, 40] # first-many, second-many
#     weight_self = 50

#     h, w, _ = img.shape
#     mat = img.copy()

#     for i in range(h):
#         # if i > 10:
#         #     continue
#         mmin, mmax = max(i-r+1, 0), min(i+r, h)
#         for j in range(w):
#             nmin, nmax = max(j-r+1, 0), min(j+r, w)
#             rect = np.zeros((mmax-mmin, nmax-nmin))
#             roi = img[mmin:mmax, nmin:nmax, :]

#             ele = img[i,j,:]
#             for m in range(mmin, mmax):
#                 for n in range(nmin, nmax):
#                     vec = ele - img[m,n,:]
#                     d = (sum(vec**2))**0.5
#                     rect[m-mmin, n-nmin] = d

#             # print 'ijmn', (i, mmin, mmax), (j, nmin, nmax) 
#             # print 'roi', roi
#             # print 'rect', rect
#             m1 = (rect<8)
#             m2 = (rect>8) & (rect<16)
#             # print m1
#             # print m2

#             c1, c2 = m1.sum(), m2.sum()
#             m1 = np.dstack([m1]*3)
#             m2 = np.dstack([m2]*3)

#             if c1 == 0 and c2 == 0:
#                 mat[i, j, :] = ele
#             elif c1 == 0 and c2 != 0:
#                 mat[i, j, :] = (roi*m2).sum(axis=(0,1))/c2
#             elif c1 !=0 and c2 == 0:
#                 mat[i, j, :] = (roi*m1).sum(axis=(0,1))/c1
#             else:
#                 w1, w2 = 0.8, 0.2
#                 mat[i, j, :] = w1*(roi*m1).sum(axis=(0,1))/c1 + w2*(roi*m2).sum(axis=(0,1))/c2
#         #     print mat[i,j,:], ele
#         #     break
#         # if i != h-1:
#         #     break
#     return mat

def test_kmeans(img):
    ## K均值聚类
    z = img.reshape((-1, 3))
    z = np.float32(z)
    criteria = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_MAX_ITER, 10, 1.0)
    ret, label, center = cv2.kmeans(z, 20, criteria, 10, cv2.KMEANS_RANDOM_CENTERS)
    center = np.uint8(center)
    res = center[label.flatten()]
    res2 = res.reshape((img.shape))
    cv2.imshow('preview', res2)
    cv2.waitKey()

def test_hsv_gradient(img):
    ## gradient test using hsv
    hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
    s = hsv[:,:,2]
    s = s[:,:,np.newaxis]
    h = np.hstack([np.diff(s, axis=1), (s[:,0,:]-s[:,-1,:])[:,np.newaxis,:]]) 
    v = np.vstack([np.diff(s, axis=0), (s[0,:,:]-s[-1,:,:])[np.newaxis,:,:]])
    edge = (h**2 + v**2)**0.5
    edge[edge<10] = 0
    cv2.imshow('preview', edge)
    cv2.waitKey()
    edge = cv2.GaussianBlur(edge, (3,3), 1)
    cv2.imshow('preview', edge)
    cv2.waitKey()

def test_detect_ui(imgname = 'base1'):
    img = cv2.imread('%s.png' % imgname)

    origin = img
    hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
    hsv_full = cv2.cvtColor(img, cv2.COLOR_BGR2HSV_FULL)
    hls = cv2.cvtColor(img, cv2.COLOR_BGR2HLS)
    hls_full = cv2.cvtColor(img, cv2.COLOR_BGR2HLS_FULL)
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    ## this is very slow
    # gray_denoised = cv2.fastNlMeansDenoising(gray, None, 20, 7, 21)
    # img_denoised = cv2.fastNlMeansDenoisingColored(img, None, 10, 10, 7, 21)

    # kernel = np.ones((5,5), np.uint8)
    kernel = cv2.getStructuringElement(cv2.MORPH_CROSS,(5,5))
    print kernel

    nochange = lambda img: img
    erosion = lambda img: cv2.erode(img, kernel, iterations=3)
    dilation = lambda img: cv2.dilate(img, kernel, iterations=3)
    opening = lambda img: cv2.morphologyEx(img, cv2.MORPH_OPEN, kernel, iterations=3)
    closing = lambda img: cv2.morphologyEx(img, cv2.MORPH_CLOSE, kernel, iterations=3)
    gradient = lambda img: cv2.morphologyEx(img, cv2.MORPH_GRADIENT, kernel, iterations=3)
    blackhat = lambda img: cv2.morphologyEx(img, cv2.MORPH_BLACKHAT, kernel, iterations=3)
    tophat = lambda img: cv2.morphologyEx(img, cv2.MORPH_TOPHAT, kernel, iterations=3)
    # laplacian = lambda img: cv2.Laplacian(gray, cv2.CV_8U)
    # sobelx = lambda img: cv2.Sobel(gray,cv2.CV_8U,1,0,ksize=3)
    # sobely = lambda img: cv2.Sobel(gray,cv2.CV_8U,0,1,ksize=3)

    revtrans = {'hsv':cv2.COLOR_HSV2BGR, 'hls':cv2.COLOR_HLS2BGR, 'hsv_full':cv2.COLOR_HSV2BGR, 'hls_full':cv2.COLOR_HLS2BGR}

    for tran in ('origin', 'gray', 'hsv', 'hsv_full', 'hls', 'hls_full', 'gray_denoised', 'img_denoised'):
        sample = locals().get(tran)
        if sample is None: continue
        # sample = cv2.GaussianBlur(sample, (3,3), 1)
        # sample = cv2.bilateralFilter(sample,9,70,70)

        for method in ('nochange', 'erosion', 'dilation', 'opening', 'closing', 'gradient', 'blackhat', 'tophat', 'laplacian', 'sobelx', 'sobely'):
            func = locals().get(method)
            if func is None: continue
            print tran, method
            mat = func(sample.copy())
            edges = cv2.Canny(mat,80,200)
            revtran = revtrans.get(tran)
            if revtran:
                mat = cv2.cvtColor(mat, revtran)
            # edges = cv2.bilateralFilter(edges, 31, 30, 30)
            cv2.imshow('preview', edges)
            cv2.waitKey()
            _, thresh = cv2.threshold(edges, 0, 255, cv2.THRESH_OTSU)
            contours, _ = cv2.findContours(thresh, cv2.RETR_CCOMP, cv2.CHAIN_APPROX_SIMPLE)
            contours.sort(key=lambda cnt: len(cnt), reverse=True)
            for cnt in contours:
                area = cv2.contourArea(cnt)
                length = cv2.arcLength(cnt,True)
                # if len(cnt) < 10:
                #     continue
                # if area < 20:# or area > 300:
                #     continue
                # if length < 100:# or length > 400:
                #     continue
                # print len(cnt), int(area), int(length)
                # epsilon = 0.2*length
                # poly = cv2.approxPolyDP(cnt,epsilon,True)
                hull = cv2.convexHull(cnt)
                hull_area = cv2.contourArea(hull)
                x,y,w,h = cv2.boundingRect(cnt)
                rect_area = float(w*h)
                if w<20 or h<20 or rect_area<100:
                    continue
                if hull_area/rect_area < 0.65:
                    continue

                cv2.drawContours(mat, [hull], 0,255,-1)
                cv2.rectangle(mat,(x,y),(x+w,y+h),(0,255,0),2)
                cnt = hull
                style = -1
                if style == 1:
                    lb,lt,rt,rb = cv.BoxPoints(cv2.minAreaRect(cnt))
                    lt = tuple(map(int, lt))
                    rb = tuple(map(int, rb))
                    cv2.rectangle(mat,lt,rb,(0,255,0),2)
                elif style == 2:
                    (x,y),radius = cv2.minEnclosingCircle(cnt)
                    center = (int(x),int(y))
                    radius = int(radius)
                    cv2.circle(mat,center,radius,(255,255,0),2)
                elif style == 3:
                    ellipse = cv2.fitEllipse(cnt)
                    cv2.ellipse(mat,ellipse,(0,255,0),2)

                # cv2.imshow('preview', mat)
                # cv2.waitKey()
            # break

            cv2.imshow('preview', mat)
            # cv2.imwrite('%s-%s-%s.png' % (imgname, tran, method), mat)
            cv2.waitKey()

def test_similar():
    from itertools import combinations
    from collections import defaultdict
    from heapq import heappush


    def sim1(img1, img2):
        h, w, d = img1.shape
        total = h*w*d
        diff = cv2.absdiff(img1, img2)
        num = (diff<10).sum()
        return num*1.0/total

    names = [os.path.join('scene', c) for c in os.listdir('scene')]
    imgs = dict(zip(names, map(cv2.imread, names)))
    diffs = defaultdict(list)

    for name1, name2 in combinations(names, 2):
        img1, img2 = imgs[name1], imgs[name2]
        similarity = sim1(img1, img2)
        # print 'diff', name1, name2, 'result is:', similarity
        heappush(diffs[name1], (-similarity, name2))
        heappush(diffs[name2], (-similarity, name1))

    for k, v in diffs.iteritems():
        print k, v[0][1], -v[0][0]

def test_find_scene():
    scenes = {}
    for s in os.listdir('txxscene'):
        if '-' in s: continue
        i = cv2.imread(os.path.join('txxscene', s), cv2.IMREAD_GRAYSCALE)
        scenes[s] = i

    # names = [os.path.join('scene', c) for c in os.listdir('scene')]
    imgs = {}
    for n in os.listdir('scene'):
        i = cv2.imread(os.path.join('scene', n), cv2.IMREAD_GRAYSCALE)
        i = cv2.resize(i, (960, 540))
        imgs[n] = i

    for name, img in imgs.iteritems():
        for scene, tmpl in scenes.iteritems():
            res = cv2.matchTemplate(img, tmpl, cv2.TM_CCOEFF_NORMED)
            min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
            if max_val < 0.6:
                continue
            x, y = max_loc
            h, w = tmpl.shape
            cv2.rectangle(img, (x, y), (x+w, y+h), 255, 2)
            print name, scene, max_val, min_val
            cv2.imshow('found', img)
            cv2.waitKey()

def build_scene_tree():
    from collections import defaultdict
    # from pprint import pprint

    class node(defaultdict):
        name = 'root'
        parent = None
        tmpl = None

        def __str__(self):
            obj = self
            names = []
            while obj.parent is not None:
                names.append(obj.name)
                obj = obj.parent
            return '-'.join(names[::-1])

    def tree():
        return node(tree)

    def walk(node, func=None, depth=0):
        if func:
            func(node)
        else:
            print ' '*depth*2, node
        for k, v in node.iteritems():
            walk(v, func, depth+1)

    scenes = tree()

    for s in os.listdir('txxscene'):
        if not s.endswith('.png'): continue
        obj = scenes
        for i in s[:-4].split('-'):
            obj[i].name = i
            obj[i].parent = obj
            obj = obj[i]
        obj.tmpl = cv2.imread(os.path.join('txxscene', s))#, cv2.IMREAD_GRAYSCALE)

    walk(scenes)

    return scenes

def test_find_scene_by_tree():
    scenes = build_scene_tree()

    # names = [os.path.join('scene', c) for c in os.listdir('scene')]
    imgs = {}
    for n in os.listdir('scene'):
        i = cv2.imread(os.path.join('scene', n))#, cv2.IMREAD_GRAYSCALE)
        i = cv2.resize(i, (960, 540))
        imgs[n] = i

    def find_match(node, img):
        # for root node
        if node.parent is None:
            for k, v in node.iteritems():
                res = find_match(v, img)
                if res is not None:
                    return res
            return node

        # find in this node
        if node.tmpl is not None:
            s_bgr = cv2.split(node.tmpl) # Blue Green Red
            i_bgr = cv2.split(img)
            weight = (0.3, 0.3, 0.4)
            resbgr = [0, 0, 0]
            for i in range(3): # bgr
                resbgr[i] = cv2.matchTemplate(i_bgr[i], s_bgr[i], cv2.TM_CCOEFF_NORMED)
            match = resbgr[0]*weight[0] + resbgr[1]*weight[1] + resbgr[2]*weight[2]

            # match = cv2.matchTemplate(img, node.tmpl, cv2.TM_CCOEFF_NORMED)
            min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(match)
            # found!
            if max_val > 0.7:
                x, y = max_loc
                h, w = node.tmpl.shape[:2]
                cv2.rectangle(img, (x, y), (x+w, y+h), 255, 2)
                # find in children
                for k, v in node.iteritems():
                    res = find_match(v, img)
                    if res is not None: 
                        return res
                return node

    for name, img in imgs.iteritems():
        cur = find_match(scenes, img)
        print '%20s %s' % (name, cur)
        cv2.imshow('img', img)
        cv2.waitKey()

def test_grid():
    m = StupidMonkey({'touch':10})
    poss = []
    while True:
        pos = m.get_touch_point()
        if not pos:
            break
        poss.append(pos)
    print 'grid point count:', len(poss)

    import cv2
    import numpy
    img = numpy.zeros((1920, 1080))
    for x,y in poss:
        img[x,y] = 255
    img = cv2.resize(img, (540, 960))
    cv2.imshow('grid', img)
    cv2.waitKey()

def _get_mini_dev():
    dev = AndroidDeviceMinicap()
    dev._adb.start_minitouch()
    time.sleep(3)
    return dev

def test_monkey():
    dev = _get_mini_dev()
    probs = {'touch':5, 'swipe':1}

    m = Monkey(probs)
    m.run(dev, package='im.yixin', maxruns=100)

def test_stupid_monkey():
    dev = _get_mini_dev()
    probs = {'touch':5}

    m = StupidMonkey(probs, 'txxscene')
    m.run(dev, package='com.netease.txx.mi')

def test_contour_monkey():
    dev = _get_mini_dev()
    probs = {'touch':5, 'swipe':1}

    m = RandomContourMonkey(probs)
    m.run(dev, package='com.netease.txx.mi')

if __name__ == '__main__':
    # test_monkey()
    # test_stupid_monkey()
    test_contour_monkey()


#!/usr/bin/env python
# coding: utf-8
#
# Usage: python -matx screenrecord -o out.avi

import os
import time
import traceback
import cv2
import numpy as np

from atx.adbkit.client import Client
from atx.adbkit.device import Device
from atx.adbkit.mixins import MinicapStreamMixin, RotationWatcherMixin

class AdbWrapper(RotationWatcherMixin, MinicapStreamMixin, Device):
    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.open_rotation_watcher(on_rotation_change=lambda v: self.open_minicap_stream())

def get_adb(host, port, serial):
    client = Client(host, port)
    if serial is None:
        serial = list(client.devices().keys())[0]
    return AdbWrapper(client, serial)

def main(serial=None, host=None, port=None, output='out.avi', scale=0.5, portrait=False, overwrite=True, verbose=True):
    if os.path.exists(output):
        print 'output file exists!'
        if overwrite:
            print 'overwriting', output
            os.remove(output)
        else:
            return

    adb = get_adb(host, port, serial)

    img = adb.screenshot_cv2()
    while img is None:
        time.sleep(1)
        img = adb.screenshot_cv2()
    if verbose:
        cv2.imshow('screen', img)

    w, h, _ = adb.display
    w, h = int(w*scale), int(h*scale)
    framesize = (w, h) if portrait else (h, w)
    fps = 24.0
    # refs http://www.fourcc.org/codecs.php
    # avaiable fourccs: XVID, MJPG
    fourcc = cv2.cv.FOURCC(*'MJPG')
    writer = cv2.VideoWriter(output, fourcc, fps, framesize)

    # video (width, height), images should be resized to fit in video frame.
    vw, vh = framesize

    tic = time.clock()
    toc = time.clock()
    while True:
        try:
            time.sleep(1.0/fps - max(toc-tic, 0))
            tic = time.clock()
            img = adb.screenshot_cv2()
            h, w = img.shape[:2]
            if h*vw == w*vh:
                h, w = vh, vw
                frame = cv2.resize(img, dsize=(w, h))
            else:
                frame = np.zeros((vh, vw, 3), dtype=np.uint8)
                sh = vh*1.0/h
                sw = vw*1.0/w
                if sh < sw:
                    h, w = vh, int(sh*w)
                else:
                    h, w = int(sw*h), vw
                left, top = (vw-w)/2, (vh-h)/2
                frame[top:top+h, left:left+w, :] = cv2.resize(img, dsize=(w, h))

            writer.write(frame)
            toc = time.clock()
            if verbose:
                cv2.imshow('screen', frame)
            cv2.waitKey(1)
        except KeyboardInterrupt:
            print 'Done'
            break
        except:
            traceback.print_exc()
            break
    writer.release()

if __name__ == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-


class BaseError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)

class WindowsAppNotFoundError(BaseError):
    pass

class ImageNotFoundError(BaseError):
    pass

class WatchTimeoutError(BaseError):
    pass

class AssertError(BaseError):
    pass

class AssertExistsError(AssertError):
    pass


#!/usr/bin/env python

# This file is part of Androguard.
#
# Copyright (C) 2012, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

import os
__dir__ = os.path.relpath(os.path.dirname(os.path.abspath(__file__))) 

import sys
sys.path.append(os.path.join(__dir__, "androguard.zip"))

from optparse import OptionParser
from xml.dom import minidom
import codecs
import string

from androguard.core import androconf
from androguard.core.bytecodes import apk


#option_0 = { 'name' : ('-i', '--input'), 'help' : 'filename input (APK or android\'s binary xml)', 'nargs' : 1 }
option_1 = {'name' : ('-f', '--format'), 
    'help': 'output format', 
    'nargs': 1,
    'default': '$package'
    }
option_2 = {
    'name': ('-v', '--version'), 
    'help':'version of the API', 
    'action': 'count' 
    }
options = [option_1, option_2]

def xml2parse(dom, strformat='$package/$activity'):
    root = dom.getElementsByTagName("manifest")[0]
    package = root.getAttribute('package')
    activity = ''
    for e in root.getElementsByTagName('activity'):
        name = e.getAttribute('android:name')
        t = e.getElementsByTagName('intent-filter')
        if t:
            activity = name
    print string.Template(strformat).safe_substitute(
            package=package, activity = activity)

def main(options, filename) :
    if filename != None :
        buff = ""

        ret_type = androconf.is_android(filename)
        if ret_type == "APK":
            a = apk.APK(filename)
            dom = a.get_android_manifest_xml()
            #buff = a.get_android_manifest_xml().toprettyxml(encoding="utf-8")
            #a.get_activities()
            xml2parse(dom)
        elif ".xml" in filename:
            ap = apk.AXMLPrinter(open(filename, "rb").read())
            buff = minidom.parseString(ap.get_buff()).toprettyxml(encoding="utf-8")
        else:
            print "Unknown file type"
            return

        #if options.output != None :
        #    fd = codecs.open(options.output, "w", "utf-8")
        #    fd.write( buff )
        #    fd.close()
        #else :
        #    print buff

    elif options.version != None :
        print "Androaxml version %s" % androconf.ANDROGUARD_VERSION

if __name__ == "__main__" :
    parser = OptionParser()
    for option in options :
        param = option['name']
        del option['name']
        parser.add_option(*param, **option)

    options, arguments = parser.parse_args()
    if len(arguments) == 0:
        sys.exit('use --help for more help')

    sys.argv[:] = arguments
    main(options, arguments[0])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from atx.device import Pattern


def test_pattern_offset():
    pt = Pattern("mixin.L50B50.png")
    assert pt.offset == (-0.5, 0.5)

    pt = Pattern("mixin.T50R50.png")
    assert pt.offset == (0.5, -0.5)

    pt = Pattern("mixin.T50R50.png")
    assert pt.offset == (0.5, -0.5)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import atexit
import argparse
import os
import time
import json
import warnings

from atx import consts
from atx import errors
from atx.base import nameddict


__dir__ = os.path.dirname(os.path.abspath(__file__))


class ExtDeprecationWarning(DeprecationWarning):
    pass

warnings.simplefilter('always', ExtDeprecationWarning)

def json2obj(data):
    data['this'] = data.pop('self', None)
    return nameddict('X', data.keys())(**data)

def center(bounds):
    x = (bounds['left'] + bounds['right'])/2
    y = (bounds['top'] + bounds['bottom'])/2
    return (x, y)


class Report(object):
    """
    Example usage:
    from atx.ext.report import Report

    Report(d)
    """
    def __init__(self, d, save_dir='report'):
        image_dir = os.path.join(save_dir, 'images')
        if not os.path.exists(image_dir):
            os.makedirs(image_dir)

        self.d = d
        self.save_dir = save_dir
        self.steps = []
        self.result = None
        self.__uia_last_position = None
        self.__last_screenshot = None
        self.__closed = False
        self.start_record()

    @property
    def last_screenshot(self):
        return self.__last_screenshot

    def _uia_listener(self, evtjson):
        evt = json2obj(evtjson)
        if evt.name != '_click':
            return
        if evt.is_before:
            self.d.screenshot()
            self.__uia_last_position = center(evt.this.bounds)
        else:
            screen_before = self._save_screenshot(self.last_screenshot)
            # FIXME: maybe need sleep for a while
            screen_after = self._save_screenshot()
            (x, y) = self.__uia_last_position
            self.add_step('click',
                screen_before=screen_before,
                screen_after=screen_after,
                position={'x': x, 'y': y})

    def patch_uiautomator(self):
        """ record steps of uiautomator """
        import uiautomator
        uiautomator.add_listener('atx-report', self._uia_listener)

    def start_record(self):
        self.start_time = time.time()

        w, h = self.d.display
        if self.d.rotation in (1, 3): # for horizontal
            w, h = h, w
        self.result = dict(device=dict(
            display=dict(width=w, height=h),
            serial=self.d.serial,
            start_time=time.strftime("%Y-%m-%d %H:%M:%S"),
            start_timestamp=time.time(),
        ), steps=self.steps)

        self.d.add_listener(self._listener, consts.EVENT_ALL) # ^ consts.EVENT_SCREENSHOT)

        atexit.register(self.close)

    def close(self):
        if self.__closed:
            return

        save_dir = self.save_dir
        data = json.dumps(self.result)
        tmpl_path = os.path.join(__dir__, 'index.tmpl.html')
        save_path = os.path.join(save_dir, 'index.html')
        json_path = os.path.join(save_dir, 'result.json')

        with open(tmpl_path) as f:
            html_content = f.read().replace('$$data$$', data)

        with open(json_path, 'wb') as f:
            f.write(json.dumps(self.result, indent=4))

        with open(save_path, 'wb') as f:
            f.write(html_content)
        self.__closed = True

    def info(self, text):
        self.steps.append({
            'time': '%.1f' % (time.time()-self.start_time,),
            'action': 'info',
            'message': text,
            'success': True,
        })

    def error(self, text, screenshot=None):
        step = {
            'time': '%.1f' % (time.time()-self.start_time,),
            'action': 'error',
            'message': text,
            'success': False,
        }   
        screen_path = 'images/error_%d.png' % time.time()
        if screenshot:
            screen_abspath = os.path.join(self.save_dir, screen_path)
            screenshot.save(screen_abspath)
            step['screenshot'] = screen_path
        self.steps.append(step)

    def add_step(self, action, **kwargs):
        kwargs['success'] = kwargs.pop('success', True)
        kwargs['time'] = round(kwargs.pop('time', time.time()-self.start_time), 1)
        kwargs['action'] = action
        self.steps.append(kwargs)

    def _save_screenshot(self, screen=None):
        if screen is None:
            screen = self.d.screenshot()
        abspath = 'images/before_%d.png' % time.time()
        relpath = os.path.join(self.save_dir, abspath)
        screen.save(relpath)
        return abspath

    def _listener(self, evt):
        d = self.d
        screen_before = 'images/before_%d.png' % time.time()
        screen_before_abspath = os.path.join(self.save_dir, screen_before)

        if evt.depth > 1: # base depth is 1
            return

        if evt.is_before: # call before function
            if evt.flag == consts.EVENT_CLICK:
                d.screenshot()
            return

        if evt.flag == consts.EVENT_CLICK:
            if self.last_screenshot: # just in case
                self.last_screenshot.save(screen_before_abspath)
            screen_after = 'images/after_%d.png' % time.time()
            d.screenshot(os.path.join(self.save_dir, screen_after))

            (x, y) = evt.args
            self.add_step('click',
                screen_before=screen_before,
                screen_after=screen_after,
                position={'x': x, 'y': y})
        elif evt.flag == consts.EVENT_SCREENSHOT:
            # keep last screenshot
            self.__last_screenshot = evt.retval
        elif evt.flag == consts.EVENT_CLICK_IMAGE:
            kwargs = {
                'success': evt.traceback is None,
                'traceback': None if evt.traceback is None else evt.traceback.stack,
            }
            # not record if image not found
            if evt.retval is None and evt.traceback is None:
                return
            
            if self.last_screenshot:
                self.last_screenshot.save(screen_before_abspath)
                kwargs['screen_before'] = screen_before
            if evt.traceback is None or not isinstance(evt.traceback.exception, IOError):
                target = 'images/target_%d.png' % time.time()
                target_abspath = os.path.join(self.save_dir, target)
                pattern = d.pattern_open(evt.args[0])
                pattern.save(target_abspath)
                kwargs['target'] = target
            if evt.traceback is None:
                screen_after = 'images/after_%d.png' % time.time()
                d.screenshot(os.path.join(self.save_dir, screen_after))
                kwargs['screen_after'] = screen_after
                kwargs['confidence'] = evt.retval.confidence
                (x, y) = evt.retval.pos
                kwargs['position'] = {'x': x, 'y': y}
            self.add_step('click_image', **kwargs)


def listen(d, save_dir='report'):
    ''' Depreciated '''
    warnings.warn(
        "Using report.listen is deprecated, use report.Report(d, save_dir) instead.", 
        ExtDeprecationWarning, stacklevel=2
    )
    Report(d, save_dir)


#!/usr/bin/env python
# coding: utf-8
#
# Usage: python -matx screen [-s 0.8]
from __future__ import absolute_import

import time
from atx import adbkit


def main(host=None, port=None, serial=None, scale=1.0, out='screenshot.png', method='minicap'):
    """
    If minicap not avaliable then use uiautomator instead

    Disable scale for now.
    Because -s scale is conflict of -s serial
    """
    start = time.time()

    client = adbkit.Client(host=host, port=port)
    device = client.device(serial)
    im = device.screenshot(scale=scale)
    im.save(out)
    print 'Time spend: %.2fs' % (time.time() - start)
    print 'File saved to "%s"' % out


if __name__ == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import cv2
import time
import numpy as np
import matplotlib.pyplot as plt
import scipy.fftpack
from cv2 import cv

def test_screenshot():
    from atx.device.android import AndroidDevice
    dev = AndroidDevice()
    print 'screen display:', dev.display
    # screen = dev._screenshot_uiauto()
    for i in range(10):
        t0 = time.clock()
        # screen = dev.screenshot()
        screen = dev._screenshot_minicap()
        print time.clock() - t0
    print dev.screenshot_method
    screen.save('tmp.png')

# def run_test():
#     d = SomeDevice()
#     d.connect() //setup info, screen
#     d.reset()

#     d.info.serial
#     d.info.wlan_ip
#     d.info.sreensize

#     d.screen = Screen(device)
#     d.screen.resolution                                 screen 1 thread
#     d.screen.orientation                                orientation 1 thread
#     d.screen.click(x, y)
#     d.screen.save('hello.png')
#     d.screen.region(l,t,w,h).save('region.png')
#     d.screen.search('xxx.png')
#     d.screen.exists('xxx.png')
#     d.screen.on()
#     d.screen.off()

#     # for android
#     d.keys.home()
#     d.keys.volup()
#     d.keys.voldown()

#     # for windows
#     d.text('hello')

#     ## short cuts
#     d.controls.install(pkg)     uiautomator.device.server.adb
#     d.controls.uninstall(pkg)
#     d.controls.startapp()
#     d.controls.stopapp()
#     d.controls.reboot()

#     # recorder.listener.start()
#     # recorder.listener.stop()
#     # recorder.listener.on_touch_down()                        listener 1 thread
#     # recorder.listener.on_touch_move()
#     # recorder.listener.on_touch_up()
#     # recorder.listener.on_click()
#     # recorder.listener.on_drag()

#     w = d.watcher()
#     w.wait('xxx.png', 5).click(x,y).expect('xxx.png', 3) --> Exception means fail
#     w.wait(1).click(x,y)
#     w.exists('xxx.png')

def test_minicap():
    from atx.device.android_minicap import AndroidDeviceMinicap

    cv2.namedWindow("preview")
    d = AndroidDeviceMinicap()

    while True:
        try:
            h, w = d._screen.shape[:2]
            img = cv2.resize(d._screen, (w/2, h/2))
            cv2.imshow('preview', img)
            key = cv2.waitKey(1)
            if key == 100: # d for dump
                filename = time.strftime('%Y%m%d%H%M%S.png')
                cv2.imwrite(filename, d._screen)
        except KeyboardInterrupt:
            break
    cv2.destroyWindow('preview')

def test_minitouch():
    from atx.device.android_minicap import SubAdb
    
    adb = SubAdb()
    adb.start_minitouch()
    adb.home()
    for pos in ((100, 200), (1000, 200), ):#(100, 1900), (1000, 1900)):
        adb.touch(*pos)
        time.sleep(1)
    for i in range(10):
        adb.swipe(100, 100, 500, 100)
        time.sleep(1)
        adb.swipe(500, 100, 100, 100)
        time.sleep(1)
    return adb


if __name__ == '__main__':
    # test_screenshot()
    test_minicap()
    # adb = test_minitouch()


#-*- encoding: utf-8 -*-

import os
import sys
import traceback
import Tkinter as tk
import tkSimpleDialog
import tkMessageBox

mswindows = (sys.platform == "win32")

import win32api
import win32con
import win32gui

from atx.device.android import AndroidDevice
from atx.device.windows import WindowsDevice
from atx.record.android import AndroidRecorder
from atx.record.windows import WindowsRecorder

__dir__ = os.path.dirname(os.path.abspath(__file__))

class SystemTray(object):
    def __init__(self, parent, name, commands=None, icon_path=None):
        self.parent = parent
        self.name = name
        self.WM_NOTIFY = win32con.WM_USER+20

        wndproc = {
            win32con.WM_DESTROY: self.on_destroy,
            win32con.WM_COMMAND: self.on_command,
            self.WM_NOTIFY: self.on_tray_notify,
        }

        wc = win32gui.WNDCLASS()
        wc.hInstance = hinst = win32api.GetModuleHandle(None)
        wc.lpszClassName = name.title()
        wc.lpfnWndProc = wndproc
        try:
            class_atom = win32gui.RegisterClass(wc)
        except:
            pass
        self.hwnd = win32gui.CreateWindow(wc.lpszClassName, "", win32con.WS_POPUP, 0,0,1,1, 0, 0, hinst, None)
        win32gui.UpdateWindow(self.hwnd)

        if icon_path is not None and os.path.isfile(icon_path):
            icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
            hicon = win32gui.LoadImage(None, icon_path, win32con.IMAGE_ICON, 0, 0, icon_flags)
        else:
            shell_dll = os.path.join(win32api.GetSystemDirectory(), "shell32.dll")
            large, small = win32gui.ExtractIconEx(shell_dll, 19, 1) #19 or 76
            hicon = small[0]
            win32gui.DestroyIcon(large[0])
        self.hicon = hicon

        flags = win32gui.NIF_ICON | win32gui.NIF_MESSAGE | win32gui.NIF_TIP | win32gui.NIF_INFO
        nid = (self.hwnd, 0, flags, self.WM_NOTIFY, self.hicon, self.name)
        win32gui.Shell_NotifyIcon(win32gui.NIM_ADD, nid)

        self.next_command_id = 1000
        self.commands = {}
        self.register_command('Exit', lambda:win32gui.DestroyWindow(self.hwnd))
        if commands is not None:
            for n, f in commands[::-1]:
                self.register_command(n, f)

    def balloon(self, msg, title=""):
        flags = win32gui.NIF_ICON | win32gui.NIF_MESSAGE | win32gui.NIF_TIP | win32gui.NIF_INFO
        nid = (self.hwnd, 0, flags, self.WM_NOTIFY, self.hicon, self.name, msg, 300, title)
        win32gui.Shell_NotifyIcon(win32gui.NIM_MODIFY, nid)

    def register_command(self, name, func):
        cid = self.next_command_id
        self.next_command_id += 1
        self.commands[cid] = (name, func)
        return cid

    def on_destroy(self, hwnd, msg, wp, lp):
        win32gui.Shell_NotifyIcon(win32gui.NIM_DELETE, (self.hwnd, 0))
        # win32gui.PostMessage(self.parent, win32con.WM_CLOSE, 0, 0)
        return True

    def on_command(self, hwnd, msg, wp, lp):
        print 1111111
        cid = win32api.LOWORD(wp)
        if not self.commands.get(cid):
            print "Unknown command -", cid
            return
        _, func = self.commands[cid]
        try:
            func()
        except Exception as e:
            traceback.print_exc()
        return True

    def on_tray_notify(self, hwnd, msg, wp, lp):
        if lp == win32con.WM_LBUTTONUP:
            # print "left click"
            # win32gui.SetForegroundWindow(self.hwnd)
            pass
        elif lp == win32con.WM_RBUTTONUP:
            # print "right click"
            menu = win32gui.CreatePopupMenu()
            for cid in sorted(self.commands.keys(), reverse=True):
                name, _ = self.commands[cid]
                win32gui.AppendMenu(menu, win32con.MF_STRING, cid, name)

            pos = win32gui.GetCursorPos()
            win32gui.SetForegroundWindow(self.hwnd)
            win32gui.TrackPopupMenu(menu, win32con.TPM_LEFTALIGN, pos[0], pos[1], 0, self.hwnd, None)
            win32gui.PostMessage(self.hwnd, win32con.WM_NULL, 0, 0)
        return True

    # def create_menu(self, menu, menu_options):
    #     for option_text, option_icon, option_action, option_id in menu_options[::-1]:
    #         if option_icon:
    #             option_icon = self.prep_menu_icon(option_icon)
            
    #         if option_id in self.menu_actions_by_id:                
    #             item, extras = win32gui_struct.PackMENUITEMINFO(text=option_text,
    #                                                             hbmpItem=option_icon,
    #                                                             wID=option_id)
    #             win32gui.InsertMenuItem(menu, 0, 1, item)
    #         else:
    #             submenu = win32gui.CreatePopupMenu()
    #             self.create_menu(submenu, option_action)
    #             item, extras = win32gui_struct.PackMENUITEMINFO(text=option_text,
    #                                                             hbmpItem=option_icon,
    #                                                             hSubMenu=submenu)
    #             win32gui.InsertMenuItem(menu, 0, 1, item)

class SelectDeviceDialog(tkSimpleDialog.Dialog):
    def body(self, master):
        print 'SelectDeviceDialog'
        box = tk.Frame(master)
        self.btn_and = tk.Button(box, text='Android', command=self.show_android_device)
        self.btn_and.pack(side=tk.LEFT, padx=10)
        self.btn_win = tk.Button(box, text='Windows', command=self.show_windows_device)
        self.btn_win.pack(side=tk.LEFT, padx=10)
        box.pack()
        self.listbox = tk.Listbox(master, width=50, height=10, selectmode=tk.SINGLE)
        self.listbox.pack()

        self.listbox.bind('<Double-Button-1>', self.ok)

        self.update_idletasks()
        self.show_android_device()

    def apply(self, event=None):
        sel = self.listbox.curselection()
        if not sel:
            return
        idx = sel[0]
        self.result = self.devices[idx]
        print self.result

    def show_android_device(self):
        self.btn_and.config(relief=tk.SUNKEN, state=tk.ACTIVE)
        self.btn_win.config(relief=tk.RAISED, state=tk.NORMAL)
        self.device_class = 'android'
        self.devices = ['sdfsfsf xiaomi IV', 'sdfsdfsf hasdf wx']
        self.listbox.delete(0, tk.END)
        for d in self.devices:
            self.listbox.insert(tk.END, d)

    def show_windows_device(self):
        self.btn_win.config(relief=tk.SUNKEN, state=tk.ACTIVE)
        self.btn_and.config(relief=tk.RAISED, state=tk.NORMAL)
        self.device_class = 'windows'
        self.devices = ['window-%d' % i for i in range(100)]
        self.listbox.delete(0, tk.END)
        for d in self.devices:
            self.listbox.insert(tk.END, d)

class RecorderGUI(object):
    def __init__(self, device=None):
        self._device = device
        self._recorder = None
        self.recording = False
        self._root = root = tk.Tk()

        root.protocol("WM_DELETE_WINDOW", self.destroy)

        self.tray = None
        def calllater():
            self.create_tray()

        tk.Button(root, text='choose', command=self.choose_device).pack()

        root.after(200, calllater)

        # no window for now.
        root.withdraw()

        # need to handle device event

    def create_tray(self):
        icon_path = os.path.join(__dir__, 'static', 'recorder.ico')
        commands  = [
            ('Choose Device', self.choose_device),
            ("Start Record", self.start_record),
            ("Stop Record", self.stop_record),
        ]
        tray = SystemTray(self._root.winfo_id(), "recorder", commands, icon_path)
        self.tray = tray
        tray.balloon('hello')
        win32gui.PumpMessages()
        self.tray = None

    def destroy(self):
        print "root destroy"
        if self._recorder is not None:
            self._recorder.stop()
        self._root.destroy()
        win32api.PostQuitMessage(0)

    def mainloop(self):
        try:
            self._root.mainloop()
        except KeyboardInterrupt:
            self.destroy()

    def choose_device(self, event=None):
        print 1111111, self.tray
        win32gui.PostMessage(self.tray.hwnd, win32con.WM_CLOSE, 0, 0)
        # self.tray.destroy()
        # self._root.deiconify()
        d = SelectDeviceDialog(self._root)
        self.create_tray()
        print d.result

    def start_record(self):
        if not self.check_recorder():
            return
        self._recorder.run()

    def stop_record(self):
        if not self.check_recorder():
            return
        self._recorder.stop()

    def check_recorder(self):
        if self._device is None:
            print "No device choosen."
            self._recorder = None
            return False

        if self._recorder is None:
            print "init recorder", type(self._device)
            if isinstance(self._device, WindowsDevice):
                record_class = WindowsRecorder
            elif isinstance(self._device, (AndroidDevice, )):
                record_class = AndroidRecorder
            else:
                print "Unknown device type", type(self._device)
                return False
            self._recorder = record_class(self._device)
        return True

def main():
    w = RecorderGUI()
    w.mainloop()  

if __name__ == '__main__':
    main()
